// (c) Computall Services 1999
// url:  http://www.ncf.ca/~aa571

// this software is produced as a sample piece of code to exercise the
// functions contained in cdg_lib.

// permission is granted for free use this demo software to develop 
// applications using cdg_lib as long as the applications are given away
// for free and are not sold.

#include <stdio.h>

#pragma comment (lib, "cdg_lib.lib")

// the library is made up of functions in three categories
// 1) interleaving/deinterleaving
// 2) error correction
// 3) error coding

// INTERLEAVING
// there are two functions, cdg_interleave and cdg_deinterleave
// de-interleave one pack
// the function deinterleave takes a single pointer to a string of
// 24 bytes and returns the deinterleaved 24 bytes in the same place
__declspec(dllimport) int cdg_deinterleave(unsigned char *);
// interleave one pack 
// the function interleave takes a single pointer to a string of
// 24 bytes and returns the interleaved 24 bytes in the same place
__declspec(dllimport) int cdg_interleave(unsigned char *);

// ERROR CORRECTING
// error correct pack header
// the function cdg_rs_2_4_decode takes a single pointer to a string of
// 24 bytes and returns the 24 bytes in the same place with the 4 header
// bytes error corrected
__declspec(dllimport) int cdg_rs_2_4_decode(unsigned char *);
// error correct pack
// the function cdg_rs_2_4_decode takes a single pointer to a string of
// 24 bytes and returns the 24 bytes in the same place with the 24 pack
// bytes error corrected
__declspec(dllimport) int cdg_rs_20_24_decode(unsigned char *);
// the function temp_cdg_rs_20_24 is a temporary function that will return
// a 0 if the 4 bytes of the header of a pack are correct, and will return
// a 1 if they are not 
__declspec(dllimport) int temp_cdg_rs_2_4(unsigned char *);

//ERROR CODING
// generate rs codes for pack header
// the function cdg_rs_2_4_code takes a single pointer to a string of 
// 24 bytes and returns a pack with the 4 bytes of header error protected
__declspec(dllimport) int cdg_rs_2_4_code(unsigned char *);
// error correct pack
// the function cdg_rs_20_42_code takes a single pointer to a string of 
// 24 bytes and returns a pack with the 24 bytes error protected
__declspec(dllimport) int cdg_rs_20_24_code(unsigned char *);

// initialize the rs lut values
// the function cdg_rs_lut_init is a simple function that needs to be called
// the first time the dll is used so that it can initialize the rs tables
__declspec(dllimport) void cdg_rs_lut_init(void);




int  main (void)
{
unsigned char d[25];
unsigned char buffer[25];
int i;

FILE *in,*out;


printf("CD+G Library test - version 0.1 - Copyright 1999 Computall Services\n\n");

// before doing any calls into the cdg_lib, first call the
// lut init function
printf("initializing the reed solomon look up tables: ");
cdg_rs_lut_init();
printf("done\n");

// demonstrate the deinterleaving and error correcting
// use the file from a toshiba drive that has not been
// deinterleaved and not error corrected
if ((in = fopen("toshcdg.cdg", "rb"))== NULL)
	{
	printf("Cannot open toshcdg.cdg\n");
	return 1;
	}

if ((out = fopen("toshdein.cdg", "w+b" )) == NULL)
	{
	printf("Cannot open toshdein.cdg\n");
	return 1;
	}

printf("de-interleaving packs from file: ");

d[0] = fgetc(in) & 0x3f;
while (!feof(in))
	{
	// read in one pack
	for (i=1;i<24;i++)
		d[i] = fgetc(in) & 0x3f;

	// de-interleave one pack
	cdg_deinterleave(d);

	/*
	// leave this code commented out until we have this routine
	// just use the temporary rs_2_4 routine to check for errors
	// instead of being able to correct them. only errors in the 
	// header may be detected, and any errors in the remainder of
	// the pack will be NOT found and will be saved to the output
	// file unfixed

	// error correct pack header
	cdg_rs_2_4_decode(d);
	// error correct pack
	cdg_rs_20_24_decode(d);
	*/

	if (temp_cdg_rs_2_4(d)==0)
		for (i=0;i<24;i++)
			buffer[i] = d[i];
	else
		for (i=0;i<24;i++)
			buffer[i] = 0;
	// output one pack
	fwrite(buffer, 1, 24, out);

	// get the first byte of next pack
	d[0] = fgetc(in) & 0x3f;
	} // end while

fclose(in);
fclose(out);
printf("done\n");

// now demonstrate the error coding and interleaving
// use the sample cdg file grabbed from a plextor drive
// so it is already been error corrected so it should be
// clean data

if ((in = fopen("plexcdg.cdg", "rb"))== NULL)
	{
	printf("Cannot open plexcdg.cdg\n");
	return 1;
	}

if ((out = fopen("cdgerint.cdg", "w+b" )) == NULL)
	{
	printf("Cannot open cdgerint.cdg\n");
	return 1;
	}

printf("interleaving packs from file: ");
d[0] = fgetc(in) & 0x3f;
while (!feof(in))
	{
// read in one pack
	for (i=1;i<24;i++)
		d[i] = fgetc(in) & 0x3f;

// null out the rs codes to show the functions work
	d[2] = 0;
	d[3] = 0;
	d[20] = 0;
	d[21] = 0;
	d[22] = 0;
	d[23] = 0;

	// error correct pack header
	cdg_rs_2_4_code(d);

	/*
	// leave this code commented out until we have this routine
	// error correct pack
	cdg_rs_20_24_code(d);
	*/

	// interleave one pack
	cdg_interleave(d);

	// write pack to file
	fwrite(d, 1, 24, out);

	// get first byte of new pack
	d[0] = fgetc(in) & 0x3f;
	} // end while

fclose(in);
fclose(out);
printf("done\n");

return 0;
}
